@@ -185,7 +185,7 @@ def load_delay_filter_and_write(datafile_list, baseline_list=None, calfile_list=
             df.write_filtered_data(res_outfilename=res_outfilename, CLEAN_outfilename=CLEAN_outfilename,
                                    filled_outfilename=filled_outfilename, partial_write=Nbls_per_load < len(baseline_list),
                                    clobber=clobber, add_to_history=add_to_history,
-                                   extra_attrs={'Nfreqs': df.Nfreqs, 'freq_array': df.hd.freq_array})
+                                   extra_attrs={'Nfreqs': df.Nfreqs, 'freq_array': df.hd.freq_array, 'channel_width': df.hd.channel_width})
             df.hd.data_array = None  # this forces a reload in the next loop
 
 
