@@ -226,6 +226,7 @@ def history_string(notes=""):
     return history
 
 
+# XXX change df/ f0 for freqs directly
 def fft_dly(data, df, wgts=None, f0=0.0, medfilt=False, kernel=(1, 11), edge_cut=0):
     """Get delay of visibility across band using FFT and Quinn's Second Method to fit the delay and phase offset.
     Arguments:
@@ -242,10 +243,12 @@ def fft_dly(data, df, wgts=None, f0=0.0, medfilt=False, kernel=(1, 11), edge_cut
     """
     # setup
     Ntimes, Nfreqs = data.shape
+    # XXX prefer saving compute by not using wgts at all
     if wgts is None:
         wgts = np.ones_like(data, dtype=np.float32)
 
     # smooth via median filter
+    # XXX is this mode ever used?
     if medfilt:
         data = copy.deepcopy(data)  # this prevents filtering of the original input data
         data.real = signal.medfilt(data.real, kernel_size=kernel)
@@ -253,6 +256,7 @@ def fft_dly(data, df, wgts=None, f0=0.0, medfilt=False, kernel=(1, 11), edge_cut
 
     # fft w/ wgts
     dw = data * wgts
+    # XXX is edge_cut ever used?
     if edge_cut > 0:
         assert 2 * edge_cut < Nfreqs - 1, "edge_cut cannot be >= Nfreqs/2 - 1"
         dw = dw[:, edge_cut:(-edge_cut + 1)]
@@ -265,6 +269,7 @@ def fft_dly(data, df, wgts=None, f0=0.0, medfilt=False, kernel=(1, 11), edge_cut
     inds, bin_shifts, peaks, interp_peaks = interp_peak(vfft)
     dlys = (fftfreqs[inds] + bin_shifts * dtau).reshape(-1, 1)
 
+    # XXX if not using phase offset, should have mode to not compute it?
     # Now that we know the slope, estimate the remaining phase offset
     freqs = np.arange(Nfreqs, dtype=data.dtype) * df + f0
     fSlice = slice(edge_cut, len(freqs) - edge_cut)
@@ -280,6 +285,13 @@ def fft_dly(data, df, wgts=None, f0=0.0, medfilt=False, kernel=(1, 11), edge_cut
     return dlys, offset
 
 
+def quinn_tau(x):
+    '''Quinn subgrid interpolation parameter (see https://ieeexplore.ieee.org/document/558515)'''
+    t = .25 * np.log(3 * x**2 + 6 * x + 1)
+    t -= 6**.5 / 24 * np.log((x + 1 - (2 / 3)**.5) / (x + 1 + (2 / 3)**.5))
+    return t
+
+
 def interp_peak(data, method='quinn', reject_edges=False):
     """
     Spectral interpolation for finding peak and amplitude of data along last axis.
@@ -321,13 +333,10 @@ def interp_peak(data, method='quinn', reject_edges=False):
                 dabs[i, ncut:] = 0.0
 
     # get argmaxes along last axis
-    if method == 'quinn':
-        indices = np.argmax(dabs, axis=-1)
-    elif method == 'quadratic':
-        indices = np.argmax(dabs, axis=-1)
-    else:
+    if method not in ('quinn', 'quadratic'):
         raise ValueError("'{}' is not a recognized peak interpolation method.".format(method))
 
+    indices = np.argmax(dabs, axis=-1)
     peaks = data[range(N1), indices]
 
     # calculate shifted peak for sub-bin resolution
@@ -336,19 +345,15 @@ def interp_peak(data, method='quinn', reject_edges=False):
     k2 = data[range(N1), (indices + 1) % N2]
 
     if method == 'quinn':
-        def tau(x):
-            t = .25 * np.log(3 * x ** 2 + 6 * x + 1)
-            t -= 6 ** .5 / 24 * np.log((x + 1 - (2. / 3.) ** .5) / (x + 1 + (2. / 3.) ** .5))
-            return t
-
         alpha1 = (k0 / k1).real
         alpha2 = (k2 / k1).real
         delta1 = alpha1 / (1 - alpha1)
         delta2 = -alpha2 / (1 - alpha2)
-        d = (delta1 + delta2) / 2 + tau(delta1 ** 2) - tau(delta2 ** 2)
+        d = (delta1 + delta2) / 2 + quinn_tau(delta1 ** 2) - quinn_tau(delta2 ** 2)
         d[~np.isfinite(d)] = 0.
 
-        ck = np.array([np.true_divide(np.exp(2.0j * np.pi * d) - 1, 2.0j * np.pi * (d - k),
+        numerator_ck = np.exp(2.0j * np.pi * d) - 1
+        ck = np.array([np.true_divide(numerator_ck, 2.0j * np.pi * (d - k),
                                       where=~(d == 0)) for k in [-1, 0, 1]])
         rho = np.abs(k0 * ck[0] + k1 * ck[1] + k2 * ck[2]) / np.abs(np.sum(ck ** 2))
         rho[d == 0] = np.abs(k1[d == 0])
@@ -706,7 +711,7 @@ def get_sun_alt(jds, latitude=-30.721526120689507, longitude=21.428303826863015)
     a = crd.AltAz(location=e)
 
     # get Sun locations
-    alts = np.array(list(map(lambda t: crd.get_sun(Time(t, format='jd')).transform_to(a).alt.value, jds)))
+    alts = np.array([crd.get_sun(Time(t, format='jd')).transform_to(a).alt.value for t in jds])
 
     if array:
         return alts
@@ -938,9 +943,9 @@ def chisq(data, model, data_wgts=None, gains=None, gain_flags=None, split_by_ant
     elif (chisq_per_ant is None) ^ (nObs_per_ant is None):
         raise ValueError('Both chisq_per_ant and nObs_per_ant must be specified or nor neither can be.')
 
-    # if data_wgts is unspecified, make it all 1.0s.
+    # if data_wgts is unspecified, make it 1.0
     if data_wgts is None:
-        data_wgts = {bl: np.ones_like(data[bl], dtype=float) for bl in data.keys()}
+        data_wgts = {bl: 1.0 for bl in data.keys()}
 
     # Expand model to include all bl in reds, assuming that model has the first bl in the redundant group
     if reds is not None:
@@ -967,7 +972,12 @@ def chisq(data, model, data_wgts=None, gains=None, gain_flags=None, split_by_ant
             if gain_flags is not None:
                 wgts = data_wgts[bl] * ~(gain_flags[ant1]) * ~(gain_flags[ant2])
             else:
-                wgts = copy.deepcopy(data_wgts[bl])
+                wgts = data_wgts[bl]
+
+            # convert wgts into nObs, handling the case where wgts is a scaler because data_wgts is None
+            if np.isscalar(wgts):
+                wgts = wgts * np.ones(data[bl].shape, dtype=float)
+            nObs_from_wgts = np.array(wgts > 0, dtype=int)
 
             # calculate chi^2
             chisq_here = np.asarray(np.abs(model_here - data[bl]) ** 2 * wgts, dtype=np.float64)
@@ -975,11 +985,11 @@ def chisq(data, model, data_wgts=None, gains=None, gain_flags=None, split_by_ant
                 if ap1 in chisq:
                     assert ap1 in nObs
                     chisq[ap1] = chisq[ap1] + chisq_here
-                    nObs[ap1] = nObs[ap1] + (wgts > 0)
+                    nObs[ap1] += nObs[ap1] + nObs_from_wgts
                 else:
                     assert ap1 not in nObs
                     chisq[ap1] = copy.deepcopy(chisq_here)
-                    nObs[ap1] = np.array(wgts > 0, dtype=int)
+                    nObs[ap1] = nObs_from_wgts
             else:
                 chisq += chisq_here
                 nObs += (wgts > 0)
@@ -989,11 +999,11 @@ def chisq(data, model, data_wgts=None, gains=None, gain_flags=None, split_by_ant
                 if ant in chisq_per_ant:
                     assert ant in nObs_per_ant
                     chisq_per_ant[ant] = chisq_per_ant[ant] + chisq_here
-                    nObs_per_ant[ant] = nObs_per_ant[ant] + (wgts > 0)
+                    nObs_per_ant[ant] = nObs_per_ant[ant] + nObs_from_wgts
                 else:
                     assert ant not in nObs_per_ant
                     chisq_per_ant[ant] = copy.deepcopy(chisq_here)
-                    nObs_per_ant[ant] = np.array(wgts > 0, dtype=int)
+                    nObs_per_ant[ant] = nObs_from_wgts
 
     return chisq, nObs, chisq_per_ant, nObs_per_ant
 
@@ -1368,9 +1378,9 @@ def red_average(data, reds=None, bl_tol=1.0, inplace=False,
                 if len(blinds) == 0:
                     blinds = data.antpair2ind(reverse_bl(blk))
                     davg = np.conj(davg)
-                data.data_array[blinds, 0, :, polind] = davg
-                data.flag_array[blinds, 0, :, polind] = favg
-                data.nsample_array[blinds, 0, :, polind] = navg
+                data.data_array[blinds, :, polind] = davg
+                data.flag_array[blinds, :, polind] = favg
+                data.nsample_array[blinds, :, polind] = navg
                 data.integration_time[blinds] = iavg
 
     # select out averaged bls
